/* BaseType.h 0.12 95/09/18 Copyright (C) 1993,1994,1995. David A. Clunie. All rights reserved. */
#ifndef __Header_Types__
#define __Header_Types__

#include <string.h>

// Don't use the name "Types.h" ... already used by Symantec libraries

enum Endian { LittleEndian, BigEndian };
enum VRType {ImplicitVR,ExplicitVR};

#ifdef __SC__
#ifdef THINK_CPLUS
#define	NOBOOLEANTYPE
#endif /* THINK_CPLUS */
#endif /* __SC__ */

#ifdef __BORLANDC__
#define	NOBOOLEANTYPE
#define	NOBOOLEANVALUES
#endif /* __SC__ */

#ifdef NOBOOLEANTYPE
typedef short int bool;
#endif // NOBOOLEANTYPE

#ifdef NOBOOLEANVALUES
const bool false = 0;
const bool true = 1;
#endif // NOBOOLEANVALUES

typedef unsigned short	Uint16;
typedef unsigned long	Uint32;
typedef signed short	Int16;
typedef signed long	Int32;
typedef float		Float32;

// The following types may be assigned and manipulated, or used
// in arrays or expressions, as if they were basic types.
//
// In particular sizeof(Uint16_L)==2, sizeof(Uint32_L)==4, etc.
//
// Note however that the assignment and increment operators
// like += and ++, are not implemented.

template <class T,int Hi,int Lo>
class Word_Base {
private:
	unsigned char bytes[((Hi>Lo)?Hi-Lo:Lo-Hi)+1];
public:
	Word_Base(void)
		{
			*this=0;
		}

	Word_Base(T word)
		{
			*this=word;
		}

	Word_Base<T,Hi,Lo>& operator=(T word)
		{
			unsigned index=Lo;
			while (1) {
				bytes[index]=(unsigned char)word;
				word>>=8;
				if (index == Hi) break;
				index+=(Lo>Hi)?-1:1;
			};
			return *this;
		}

	Word_Base<T,Hi,Lo>& operator=(const Word_Base<T,Hi,Lo>& i)
		{
			// NB. be very careful about the size here !!!!
			memcpy((char *)this,(const char *)&i,sizeof *this);
			return *this;
		}

	operator T() const
		{
			T word=0;
			unsigned index=Hi;
			while (1) {
				word|=bytes[index];
				if (index == Lo) break;
				index+=(Hi>Lo)?-1:1;
				word<<=8;
			};
			return word;
		}
};

typedef Word_Base<Uint16,1,0> Uint16_L;
typedef Word_Base<Uint16,0,1> Uint16_B;
typedef Word_Base<Uint32,3,0> Uint32_L;
typedef Word_Base<Uint32,0,3> Uint32_B;

#endif // __Header_Types__
