static char *CopyrightIdentifier(void) { return "@(#)ge98ex9t.cc 0.12 95/09/18 Copyright (C) 1993,1994,1995. David A. Clunie. All rights reserved."; }

#include <stdlib.h>
#include <fstream.h>
#include <strstream.h>
#include <iomanip.h>

#include <unistd.h>	// for read(2)

extern "C" {
int symlink(const char*,const char *);
}

#include <fcntl.h>

#include <sys/types.h>
#include <sys/stat.h>
#include <sys/ioctl.h>
#include <sys/mtio.h>

#ifdef NEEDMISSINGPROTOTYPES
extern "C" {
int ioctl(int fd, int request, caddr_t arg);
}
#endif /* NEEDMISSINGPROTOTYPES */

#include <string.h>
#include <ctype.h>	// for isspace()

#include <errno.h>	// for mkdir() return values
extern int sys_nerr;
extern char *sys_errlist[];

void
makeDirsOnPath(const char *p)
{
	char *path = new char [strlen(p)+1];
	strcpy(path,p);
	char *delim=path;
	while (delim=strchr(++delim,'/')) {
		*delim=0;
		if (mkdir(path,0755) == -1 && errno != EEXIST) {
			cerr << "Can't mkdir " << path << "\n" << flush;
			exit(1);
		}
		*delim='/';
	}
	delete[] path;
}

unsigned short
getWord(const char *str)
{
	// Data General is big endian ...
	return ((unsigned char *)str)[0]<<8|((unsigned char *)str)[1];
}

char *
getTapeNumber(const char *buf)
{
	ostrstream ost;
	ost << "TAPEDIR."<< setw(4) << setfill('0') << getWord(buf+36) << ends;
	return ost.str();
}

char *
getPathName(const char *buf)
{
	const char * glblhdr=buf;

	const char * examhdr;
	if (getWord(glblhdr+68)) examhdr=buf+(getWord(glblhdr+68)-1)*2;
	else examhdr=buf+512;

	const char * img1hdr;
	if (getWord(glblhdr+70)) img1hdr=buf+(getWord(glblhdr+70)-1)*2;
	else img1hdr=buf+1024;

	unsigned exam =  getWord(img1hdr+4);
	unsigned prs  =  getWord(img1hdr+20);

	char *ext;
	// filetype in image header 1
	unsigned filetype=getWord(img1hdr+434);
	switch (filetype) {
		case 1:	ext="YP"; break;	// prospective
		case 2:	ext="YV"; break;	// scout
		case 3:	ext="YR"; break;	// retrospective
		case 4:	ext="YG"; break;	// segmented
		case 5:	ext="YS"; break;	// screensave
		case 6:	ext="YL"; break;	// plot
		// what about reformatted images ? YF
		default: {
				cerr << "Warning: Unrecognized file type "
				     << filetype << "\n" << flush;
				ostrstream ost;
				ost << "External/" << ends;
				return ost.str();
			}
			break;
	}

	// patient id in exam header
	char id[13];
	// strncpy(id,examhdr+22,12); id[12]=0;
	const char *from=examhdr+22;
	const char *hdrlast=examhdr+22+11;
	char *to=id;

	while (from <= hdrlast && *from) {
		if (isspace(*from)) {
			if (to == id) ++from;	// skip leading spaces
			else break;		// stop on non-leading space
		}
		else *to++=*from++;
	}
	*to=0;

	char *pathname;
	{
		ostrstream ost;
		ost	<< id  << "/"
			<< setw(5) << setfill('0') << exam
			<< "/"
			<< setw(2) << setfill('0') << prs
			<< "/"
			<< ext << "/"
			<< ends;

		pathname=ost.str();
	}

	return pathname;
}

unsigned short
getImageNumber(const char *buf)
{
	const char * glblhdr=buf;
	const char * img1hdr;
	if (getWord(glblhdr+70)) img1hdr=buf+(getWord(glblhdr+70)-1)*2;
	else img1hdr=buf+1024;
	return getWord(img1hdr+94);
}

char *
getExternalFileName(const char *buf)
{
	const char * glblhdr=buf;
	char * filename = new char[14];
	strncpy(filename,glblhdr+32,13); filename[13]=0;
	return filename;
}

char *
getSimplePath(const char *buf,const char *pathname)
{
	char *simplepath;
	{
		ostrstream ost;
		ost 	<< pathname
			<< setw(2) << setfill('0') << getImageNumber(buf)
			<< ends;

		simplepath=ost.str();
	}
	return simplepath;
}

char *
getExternalPath(const char *buf,const char *pathname)
{
	char *externalpath;
	char *externalname=getExternalFileName(buf);
	{
		ostrstream ost;
		ost 	<< pathname
			<< externalname
			<< ends;

		externalpath=ost.str();
	}
	delete[] externalname;
	return externalpath;
}

char *
getLinkPath(const char *outname)
{
	char *linkpath;
	{
		ostrstream ost;
		ost 	<< "../../../../"
			<< outname
			<< ends;

		linkpath=ost.str();
	}
	return linkpath;
}

int
command(int fd,short op,daddr_t count)
{
	struct mtop mt_command;
	struct mtget mt_status;
	mt_command.mt_op = op;
	mt_command.mt_count = count;
	if (ioctl(fd, MTIOCTOP, (caddr_t)&mt_command) == -1) {
		cerr << "ioctl command fail"
		     << " op=" << op << " count=" << count
		     << " perror=" << errno
		     << " " << (errno<sys_nerr ? sys_errlist[errno] : "") << " "
		     << "\n" << flush;
		return -1;
	}
	if (ioctl(fd, MTIOCGET, (caddr_t)&mt_status) == -1) {
		cerr << "ioctl status fail"
		     << " perror=" << errno
		     << " " << (errno<sys_nerr ? sys_errlist[errno] : "") << " "
		     << "\n" << flush;
		return -1;
	}
	if (mt_status.mt_erreg) {
		cerr << "mt_status error"
		     << " mt_dsreg=" << mt_status.mt_dsreg
		     << " mt_erreg=" << mt_status.mt_erreg
		     << "\n" << flush;
		return -1;
	}
	return 0;
}


int
fsf(int fd,daddr_t count)
{
	return command(fd,MTFSF,count);
}

int
main(int argc,char **argv)
{
	char *devicename;
	static char buf[8192];
	int fd;

	devicename=argv[1];
	(void)argc;

	if ((fd=open(devicename,O_RDONLY)) == -1) {
		cerr << "Can't open " << devicename << "\n" << flush;
		exit(1);
	}

	int errormax=4;
	int errorcount=0;
	int markcount=0;
	int lng=0;
	ofstream outstream;
	char *outname=0;
	int doingfirstfile=1;
	while (markcount < 2) {
		int num=read(fd,buf,8192);
		switch (num) {
			case -1:
				cerr << "Read Error\n" << flush;
				if (doingfirstfile && lng == 0) {
					cerr << "At start of tape"
					     << " - probably wrong density"
					     << " - not attempting recovery\n"
					     << flush;
					exit(1);
				}
				++errorcount;
				if (outname) {
					cerr << "Skipping file "
					     << outname
					     << " which is bad\n" 
					     << flush;
					outstream.close();
					delete[] outname;
					outname=0;
					lng=0;
				}
				if (errorcount > errormax || fsf(fd,1) == -1) {
					cerr << "Giving up after "
					     << errorcount
					     << " attempts to recover\n"
					     << flush;
					exit(1);
				}
				else {
					cerr << "Skipped to next tape mark\n" 
					     << flush;
					markcount=1;
					doingfirstfile=0;
				}
				break;
			case 0:
				if (outname) {
					outstream.close();
					delete[] outname;
					outname=0;
					cerr << "Read " << lng << " bytes\n" 
					     << flush;
					lng=0;
				}
				++markcount;
				errorcount=0;
				doingfirstfile=0;
				break;
			default:
				markcount=0;
				errorcount=0;
				lng+=num;
				if (!outname) {
					char *path;
					if (doingfirstfile) {
						outname=getTapeNumber(buf+512);
					}
					else {
						path=getPathName(buf);
						makeDirsOnPath(path);
						outname=getSimplePath(buf,path);
					}
					cerr << outname << "\n" << flush;
					outstream.open(outname);
					if (!doingfirstfile) {
						char *epath=
						    getExternalPath(buf,path);
						char *lpath=
						    getLinkPath(outname);
						symlink(lpath,epath);
						cerr << epath << "\n" << flush;
						delete[] epath;
						delete[] lpath;
					}
					delete[] path;
				}
				outstream.write(buf,num);
				if (!outstream.good()) {
					cerr << "Write error " << outname
					     << "\n" << flush;
					exit(1);
				}
				break;
		}
	}
	cerr << "End Of Media\n" << flush;
	exit(0);
}

