static char *CopyrightIdentifier(void) { return "@(#)ge98ls9t.cc 0.12 95/09/18 Copyright (C) 1993,1994,1995. David A. Clunie. All rights reserved."; }

#include <stdlib.h>
#include <iostream.h>
#include <iomanip.h>
#include <string.h>

#include "GetOpt.h"
#include "MainCmd.h"

#include "ge98df9t.h"

static char buffer[8192];
static char string[60];
static int offset;

inline short
word_at(unsigned p)
{
	unsigned char *up = (unsigned char *)buffer + 2*(offset+p);
	return up[0]<<8|up[1];	// Data General is big endian
}

#define byte_at(p)	( *(char * )(buffer + 2*(offset+p)) )
#define string_at(p)	( strncpy(string,buffer + 2*(offset+p),p##_lng),string[p##_lng]=0,string )

#define zeropad(i,l)	dec << setfill('0') << setw(l) << i << setfill(' ')

enum Verbosity { VB_BRIEF, VB_VERBOSE, VB_IDONLY, VB_TAPENUMONLY };

static void
show_image_entries(Verbosity verbosity,char type)
{
    char *s;

    switch(type) {
	case 'P':	s="Prospective"; break;
	case 'V':	s="ScoutView"; break;
	case 'R':	s="Retrospective"; break;
	case 'G':	s="Segmented Recon"; break;
	case 'S':	s="Screen Save"; break;
	case 'L':	s="Plot Data"; break;
	case 'F':	s="Reformatted"; break;
	default:	s="-unrecognized-"; break;
    }

    if (verbosity == VB_BRIEF && offset != -1) {
	cout << "        Img #   Tapefile # (" << s << " images)\n";
    }
    while (offset != -1) {
	int next_image;
	--offset;	/* offsets start from 1 not 0 !! */
	switch (verbosity) {
	    case VB_VERBOSE:
		cout << s << "Image Entry:\n\n";
		cout << "\tImage Number:\t\t\t" 		<< zeropad(word_at(GE98_TAPE_IMNUM),2) << "\n";
		cout << "\tReconstruction Number:\t\t" 	<< word_at(GE98_TAPE_IMRNUM) << "\n";
		cout << "\tReconstruction Date M/D/Y:\t" 	<< zeropad(word_at(GE98_TAPE_IMRMON),2) << "/" 
								<< zeropad(word_at(GE98_TAPE_IMRDAY),2) << "/" 
								<< zeropad(word_at(GE98_TAPE_IMRYEAR),2) << "\n";
		cout << "\tReconstruction Time hh:mm:ss\t" 	<< zeropad(word_at(GE98_TAPE_IMRHOUR),2) << ":" 
								<< zeropad(word_at(GE98_TAPE_IMRMIN),2) << ":" 
								<< zeropad(word_at(GE98_TAPE_IMRSEC),2) << "\n";
		cout << "\tImage Size (disk blks):\t\t" 	<< word_at(GE98_TAPE_IMSIZ) << "\n";
		cout << "\tArchival Date M/D/Y:\t\t" 	<< zeropad(word_at(GE98_TAPE_IMAMON),2) << "/" 
								<< zeropad(word_at(GE98_TAPE_IMADAY),2) << "/" 
								<< zeropad(word_at(GE98_TAPE_IMAYEAR),2) << "\n";
		cout << "\tFile #:\t\t\t\t" 		<< word_at(GE98_TAPE_IMFNO) << "\n";
		cout << "\tBackward link\t\t\t" 		<< word_at(GE98_TAPE_IMBCK) << "\n";
		cout << "\tNext Image Entry:\t\t" 		<< word_at(GE98_TAPE_IMNXT) << "\n";
		cout << "\n";
		break;
	    case VB_BRIEF:
		cout << "        " << type << " " << zeropad(word_at(GE98_TAPE_IMNUM),2) << "    " << word_at(GE98_TAPE_IMFNO) << "\n";
		break;
	}
	next_image=word_at(GE98_TAPE_IMNXT);
	offset=next_image;
    }
}

static void
show_position_entries(Verbosity verbosity)
{
    while (offset != -1) {
	int next_position,old_position;
	--offset;	/* offsets start from 1 not 0 !! */
	switch (verbosity) {
	    case VB_VERBOSE:
		cout << "Position Code Entry:\n\n";
		cout << "\tPosition Code:\t\t\t" 	<< zeropad(word_at(GE98_TAPE_PCCODE),2) << "\n";
		cout << "\t1st Prospective image:\t\t" 	<< word_at(GE98_TAPE_PCPRO) << "\n";
		cout << "\t1st Scout image:\t\t" 	<< word_at(GE98_TAPE_PCSCT) << "\n";
		cout << "\t1st Retrospective image:\t" 	<< word_at(GE98_TAPE_PCRET) << "\n";
		cout << "\t1st Segmented image:\t\t" 	<< word_at(GE98_TAPE_PCSEG) << "\n";
		cout << "\t1st Screen save image:\t\t" 	<< word_at(GE98_TAPE_PCSAV) << "\n";
		cout << "\t1st Plot data image:\t\t" 	<< word_at(GE98_TAPE_PCPDI) << "\n";
		cout << "\tBackward link:\t\t\t" 	<< word_at(GE98_TAPE_PCBCK) << "\n";
		cout << "\tNext position code entry:\t"	<< word_at(GE98_TAPE_PCNXT) << "\n";
		cout << "\n";
		break;
	    case VB_BRIEF:
		cout << "      Prs " << zeropad(word_at(GE98_TAPE_PCCODE),2) << "\n";
		break;
	}
	next_position=word_at(GE98_TAPE_PCNXT);
	old_position=offset;
	offset=word_at(GE98_TAPE_PCPRO);
	show_image_entries(verbosity,'P');
	offset=old_position;
	offset=word_at(GE98_TAPE_PCSCT);
	show_image_entries(verbosity,'V');
	offset=old_position;
	offset=word_at(GE98_TAPE_PCRET);
	show_image_entries(verbosity,'R');
	offset=old_position;
	offset=word_at(GE98_TAPE_PCSEG);
	show_image_entries(verbosity,'G');
	offset=old_position;
	offset=word_at(GE98_TAPE_PCSAV);
	show_image_entries(verbosity,'S');
	offset=old_position;
	offset=word_at(GE98_TAPE_PCPDI);
	show_image_entries(verbosity,'L');

	offset=next_position;
    }
}

static void
show_exam_entries(Verbosity verbosity)
{
    while (offset != -1) {
	int next_exam;
	--offset;	/* offsets start from 1 not 0 !! */
	switch (verbosity) {
	    case VB_VERBOSE:
		cout << "Exam Entry:\n\n";
		cout << "\tScan Station #:\t\t" 	<< word_at(GE98_TAPE_EXSSN) << "\n";
		cout << "\tExam #:\t\t\t" 		<< zeropad(word_at(GE98_TAPE_EXNUM),5) << "\n";
		cout << "\tScan Date M/D/Y:\t" 		<< zeropad(word_at(GE98_TAPE_EXMON),2) << "/" 
							<< zeropad(word_at(GE98_TAPE_EXDAY),2) << "/" 
							<< zeropad(word_at(GE98_TAPE_EXYEAR),2) << "\n";
		cout << "\t1st Position Entry:\t" 	<< word_at(GE98_TAPE_EX1ST) << "\n";
		cout << "\tNext Exam Entry:\t" 		<< word_at(GE98_TAPE_EXNXT) << "\n";
		cout << "\tBackward Link:\t\t" 		<< word_at(GE98_TAPE_EXBCK) << "\n";
		cout << "\n";
		break;
	    case VB_BRIEF:
	    case VB_IDONLY:
		cout << "    Exam " << zeropad(word_at(GE98_TAPE_EXNUM),5) << "\n";
		cout << "    Date " << zeropad(word_at(GE98_TAPE_EXMON),2) << "/" 
				    << zeropad(word_at(GE98_TAPE_EXDAY),2) << "/" 
				    << zeropad(word_at(GE98_TAPE_EXYEAR),2) << "\n";
		break;
	    case VB_TAPENUMONLY:
		break;
	}
	next_exam=word_at(GE98_TAPE_EXNXT);
	offset=word_at(GE98_TAPE_EX1ST);
	show_position_entries(verbosity);
	offset=next_exam;
    }
}

static void
show_patient_entries(Verbosity verbosity)
{
    while (offset != -1) {
	int next_patient;
	--offset;	/* offsets start from 1 not 0 !! */
	switch (verbosity) {
	    case VB_VERBOSE:
		cout << "Patient Entry:\n\n";
		cout << "\tPatient ID:\t\t" 		<< string_at(GE98_TAPE_PEPID) << "\n";
		cout << "\tPatient Name:\t\t" 		<< string_at(GE98_TAPE_PEPNM) << "\n";
		cout << "\t1st Exam Entry:\t\t" 	<< word_at(GE98_TAPE_PE1ST) << "\n";
		cout << "\tNext Patient Entry:\t" 	<< word_at(GE98_TAPE_PENXT) << "\n";
		cout << "\tBackward Link:\t\t" 		<< word_at(GE98_TAPE_PEBCK) << "\n";
		cout << "\n";
		break;
	    case VB_BRIEF:
	    case VB_IDONLY:
		cout << "  PatientName "	<< string_at(GE98_TAPE_PEPNM) << "\n";
		cout << "  PatientID "		<< string_at(GE98_TAPE_PEPID) << "\n";
		break;
	    case VB_TAPENUMONLY:
		break;
	}
	next_patient=word_at(GE98_TAPE_PENXT);
	offset=word_at(GE98_TAPE_PE1ST);
	show_exam_entries(verbosity);
	offset=next_patient;
    }
}

static void
show_global_entries(Verbosity verbosity)
{
    switch (verbosity) {
	case VB_VERBOSE:
	    cout << "Global Entry:\n\n";

	    cout << "\tCT System ID Field:\t" 	<< string_at(GE98_TAPE_GEMID) << "\n";
	    cout << "\tRevision Level:\t\t" 	<< word_at(GE98_TAPE_GEREV) << "\n";
	    cout << "\tAttached Flag:\t\t" 	<< word_at(GE98_TAPE_GEADF) << "\n";
	    cout << "\tHospital Name:\t\t" 	<< string_at(GE98_TAPE_GEHOS) << "\n";
	    cout << "\tSystem ID:\t\t" 		<< word_at(GE98_TAPE_GESYS) << "\n";
	    cout << "\tSystem ID created on:\t" << word_at(GE98_TAPE_GECID) << "\n";
	    cout << "\tTape ID #:\t\t" 		<< word_at(GE98_TAPE_GETID) << "\n";
	    cout << "\tTape Encoding Mode:\t" 	<< word_at(GE98_TAPE_GEDNS) << "\n";
	    cout << "\tTape Length:\t\t" 	<< word_at(GE98_TAPE_GELNG) << "\n";
	    cout << "\tUnused tape Length:\t" 	<< word_at(GE98_TAPE_GEUNL) << "\n";
	    cout << "\tData Compressed:\t" 	<< word_at(GE98_TAPE_GECMP) << "\n";
	    cout << "\tTape Full:\t\t" 		<< word_at(GE98_TAPE_GEFUL) << "\n";
	    cout << "\tCreation Date M/D/Y:\t" 	<< zeropad(word_at(GE98_TAPE_GECMON),2) << "/" 
						<< zeropad(word_at(GE98_TAPE_GECDAY),2) << "/" 
						<< zeropad(word_at(GE98_TAPE_GECYEAR),2) << "\n";
	    cout << "\tRecent Use Date M/D/Y:\t"<< zeropad(word_at(GE98_TAPE_GEUMON),2) << "/" 
						<< zeropad(word_at(GE98_TAPE_GEUDAY),2) << "/" 
						<< zeropad(word_at(GE98_TAPE_GEUYEAR),2) << "\n";
	    cout << "\t# of Images on Tape:\t" 	<< word_at(GE98_TAPE_GENUM) << "\n";
	    cout << "\t1st Patient Entry:\t" 	<< word_at(GE98_TAPE_GE1ST) << "\n";
	    cout << "\t1st Unused Entry:\t" 	<< word_at(GE98_TAPE_GEUNE) << "\n";
	    cout << "\n";
	    break;
	case VB_BRIEF:
	case VB_TAPENUMONLY:
	    cout << "Tape # " << word_at(GE98_TAPE_GETID) << " contains " << word_at(GE98_TAPE_GENUM) << " images\n";
	    break;
	case VB_IDONLY:
	    break;
    }
    offset=word_at(GE98_TAPE_GE1ST);
    show_patient_entries(verbosity);
}


int
main(int argc,char **argv)
{
	CCOMMAND(argc,argv);

	GetOpt getopt(argc,argv,"vbit");

	int c;
	int err=0;

	Verbosity verbosity=VB_BRIEF;

	while ((c=getopt()) != -1 && !err) {
		switch (c) {
			case 'v':	// Verbose
				verbosity=VB_VERBOSE;
				break;
			case 'b':	// Verbose
				verbosity=VB_BRIEF;
				break;
			case 'i':	// Verbose
				verbosity=VB_IDONLY;
				break;
			case 't':	// Verbose
				verbosity=VB_TAPENUMONLY;
				break;
			case '?':
			default:
				err=1;
		}
	}

	if (err || argc != getopt.optind) {
		cerr << "usage: " << argv[0] << " [-v|b|i|t] < filename\n";
		exit(0);
	}
	if (!cin.read(buffer,512)) {			// skip past header
		cerr << "can't read file\n";
		exit(1);
	}
	if (!cin.read(buffer,8192)) {
		cerr << "can't read file\n";
		exit(1);
	}
	offset=0;
	show_global_entries(verbosity);
	exit(0);
}

