static char *CopyrightIdentifier(void) { return "@(#)date.cc 0.12 95/09/18 Copyright (C) 1993,1994,1995. David A. Clunie. All rights reserved."; }
#include <iostream.h>

#include <ctype.h>

static inline int
isDateDelimeter(char c)
{
	return c == '-' || c == '/';
}

static inline int
isTimeDelimeter(char c)
{
	return c == ':';
}

static inline void
skipDateDelimeter(const char *&src,int &length)
{
	if (length>0 && isDateDelimeter(*src)) { --length; ++src; }
}

static inline void
skipTimeDelimeter(const char *&src,int &length)
{
	if (length>0 && isTimeDelimeter(*src)) { --length; ++src; }
}

static inline void
skipSpaces(const char *&src,int &length)
{
	while (length>0 && isspace(*src)) { --length; ++src; }
}

static inline int
extractUnsignedDecimal(const char *&src,int &length,unsigned &n)
{
	n=0;
	if (length>0 && isdigit(*src)) {
		do {
			--length; n=n*10+(*src++-'0');
		} while (length>0 && isdigit(*src));
		return 0;
	}
	else return -1;
}

static int
extractMMFromMMM(const char *&src,int &length,unsigned &mm)
{
	int bad=0;
	mm=0;
	{
		switch (tolower(*src)) {
			case 'j': 	mm=(tolower(*(src+1)) == 'a')
						? 1		// jan
						: ((tolower(*(src+2)) == 'n')
							? 6 	// jun
							: 7 	// jul
							)
						;
					break;
			case 'f':	mm=2;  break;	// feb
			case 'm': 	mm=(tolower(*(src+2)) == 'r')
						? 3	// mar
						: 5 	// may
						;
					break;
			case 'a':	mm=(tolower(*(src+1)) == 'p')
						? 4	// apr
						: 8 	// aug
						;
					break;
			case 's':	mm=9;  break;	// sep
			case 'o':	mm=10; break;	// oct
			case 'n':	mm=11; break;	// nov
			case 'd':	mm=12; break;	// dec
			default:	bad=-1; break;
		}
		length-=3;
		src+=3;
	}
	return bad;
}

static int
getDatePossibilityFromString(
	const char *&src,int &length,
	unsigned &value,int &value_ismmm)
{
	if (length) {
		if (isdigit(*src)) {
			value_ismmm=0;
			return extractUnsignedDecimal(src,length,value);
		}
		else {
			value_ismmm=1;
			return extractMMFromMMM(src,length,value);
		}
	}
	else return -1;
}


static int
getDatePossibilitiesFromString(
	const char *src,int length,
	unsigned &v1,int &v1_ismmm,
	unsigned &v2,int &v2_ismmm,
	unsigned &v3,int &v3_ismmm)
{
	skipSpaces(src,length);
	if (getDatePossibilityFromString(src,length,v1,v1_ismmm) < 0) return -1;
	skipSpaces(src,length);
	skipDateDelimeter(src,length);
	skipSpaces(src,length);
	if (getDatePossibilityFromString(src,length,v2,v2_ismmm) < 0) return -1;
	skipSpaces(src,length);
	skipDateDelimeter(src,length);
	skipSpaces(src,length);
	if (getDatePossibilityFromString(src,length,v3,v3_ismmm) < 0) return -1;
	skipSpaces(src,length);

	return (length == 0 && v1 && v2 && v3 && (v1_ismmm+v2_ismmm+v3_ismmm) <= 1);
}

static inline int
validateDateValues(unsigned dd,unsigned mm,unsigned &yyyy)
{
	if (yyyy && yyyy < 100) yyyy+=(yyyy < 70 ? 2000 : 1900);
	return yyyy && mm && mm <= 12 && dd && dd <= 31;
}

static int
getDateValuesFromStringDefaultMonthMiddleYearLast(
	const char *src,int length,
	unsigned &dd,unsigned &mm,unsigned &yyyy)
{
	unsigned v[3];
	int v_ismmm[3];
	if (getDatePossibilitiesFromString(
		src,length,
		v[0],v_ismmm[0],
		v[1],v_ismmm[1],
		v[2],v_ismmm[2]) < 0) return -1;

	int dayindex	=-1;
	int monthindex	=-1;
	int yearindex	=-1;

	unsigned i;

	for (i=0; i<3; ++i)
		if (v_ismmm[i]) {
			monthindex=i; break;
		}

	for (i=0; i<3; ++i)
		if (i != monthindex
		 && v[i] > 31) { yearindex=i; break; }

	for (i=0; i<3; ++i)
		if (i != yearindex
		 && i != monthindex
		 && v[i] > 12) { dayindex=i; break; }

	// use default assignment order
	//  - mm in middle
	//  - yy last unless known

	if (yearindex != 0 && dayindex < 0)	// default to ddmmyy
		for (i=0; i<3; ++i)
			if (i != monthindex
			 && i != yearindex) { dayindex=i; break; }

	if (monthindex < 0)
		for (i=0; i<3; ++i)
			if (i != dayindex
			 && i != yearindex) { monthindex=i; break; }

	if (yearindex < 0)
		for (i=0; i<3; ++i)
			if (i != dayindex
			 && i != monthindex) { yearindex=i; break; }

	if (dayindex < 0)			// just in case yymmdd
		for (i=0; i<3; ++i)
			if (i != monthindex
			 && i != yearindex) { dayindex=i; break; }

	// all will be assigned by now

	dd=v[dayindex];
	mm=v[monthindex];
	yyyy=v[yearindex];

	return validateDateValues(dd,mm,yyyy);
}

main()
{
	char *str[] = {
		" 1-JAN-1971",
		"2-feb-2008",
		"12-feb-85",
		"12-mar-85",
		"12-apr-85",
		"12-may-85",
		"12-jun-85",
		"12-jul-85",
		"12-aug-85",
		"12-sep-85",
		"12-oct-85",
		"12-nov-85",
		"12-dec-85",
		"2-2-1994",
		"2-2-94",
		"2-3-2",
		"2-22-2",
		"32-22-2",
		"1995-11-2",
		"1995-15-2",
		"mar-2-2",
		"mar-2-1992",
		0
	};

	char **p;
	for (p=str; *p; ++p) {
		unsigned dd;
		unsigned mm;
		unsigned yyyy;

		cerr << "for <" << *p << "> returns "
		     << getDateValuesFromStringDefaultMonthMiddleYearLast(*p,strlen(*p),dd,mm,yyyy)
		     << "\n";

		cerr << "\t dd = " << dd
		     << " mm = " << mm
		     << " yyyy = " << yyyy << "\n";
	}
}
