static char *CopyrightIdentifier(void) { return "@(#)filehist.cc 0.12 95/09/18 Copyright (C) 1993,1994,1995. David A. Clunie. All rights reserved."; }

#ifdef __GNUG__
#include <builtin.h>
#endif
#include <fstream.h>
#include <stdlib.h>         // for atoi()
#include <ctype.h>

#include "GetOpt.h"
#include "MainCmd.h"

unsigned long
readBigEndianValue(istream& istr,unsigned nbytes)
{
	unsigned char *buffer;
	buffer=new unsigned char[nbytes];
	unsigned char *ptr = buffer;
	unsigned long value = 0;

	istr.read(buffer,nbytes);

	value=0;
	while (nbytes--) {
		value<<=8;
		value|=*ptr++;
	}
	if (buffer) delete buffer;
	return value;
}

int
main(int argc,char **argv)
{
	CCOMMAND(argc,argv);

	// *************** Setup options ***************

	char *	inname = 0;

	unsigned bucketbits	= 8;
	unsigned databits	= 16;
	unsigned skipbytes	= 0;
	int showhist		= 0;
	int graphhist		= 0;
	int verbose		= 0;

	GetOpt getopt(argc,argv,"b:d:ghj:s:v");
	int c;
	int err=0;
	while ((c=getopt()) != -1 && !err) {
		switch (c) {
			case 'b':	bucketbits=atoi(getopt.optarg);
					break;
			case 'd':	databits=atoi(getopt.optarg);
					break;
			case 'g':	graphhist=1;
					break;
			case 'h':	showhist=1;
					break;
			case 's':
			case 'j':	skipbytes=atoi(getopt.optarg);
					break;
			case 'v':	verbose=1;
					break;
			case '?':
			default:
				err=1;
		}
	}

	istream *instream=&cin;

	if (!err && argc > getopt.optind) {
		if (argc <= getopt.optind+1) {
			inname=argv[getopt.optind];
			ifstream *infstream=new ifstream(inname,ios::in);
			if (!infstream->rdbuf()->is_open()) {
				cerr << "Can't open <" << inname
				     << "> for read\n" << flush;
				return 1;
			}
			else {
				instream=infstream;
			}
		}
		else err=1;
	}
	else {
		// default for some environments is unbuffered which is slow++
		// the following improves performance 25 fold with libg++ ...

		static const unsigned bufsize=4096;
		streambuf *in  = cin.rdbuf();
		static char inbuf[bufsize];
		in->setbuf(inbuf,sizeof(inbuf));
	}

	if (databits<8) {
		cerr << "Warning: assume -d was specified as bytes not bits\n";
		databits*=8;
	}

	if (databits%8) {
		cerr << "Error - data bits must be multiple of 8\n"
		     << flush;
		err=1;
	}

	if (databits < bucketbits) {
		cerr << "Error - bucket bits must be less than data bits\n"
		     << flush;
		err=1;
	}

	if (err) {
		cerr << "Usage: " << argv[0]
		     << " -b rangebits -d databits -g -h -j|s skipbytes -v"
		     << " [infile]\n"
		     << flush;
		return 1;
	}

	// *************** Actually do something ***************

	// don't assume we can seek ...

	if (verbose) {
		cerr << "Skipping " << skipbytes << " bytes\n";
	}

	{ char buffer[1]; while (skipbytes--) instream->read(buffer,1); }

	if (!instream->good()) {
		cerr << "Error - couldn't jump over header\n" << flush;
		return 1;
	}

	unsigned nbuckets=1<<bucketbits;
	unsigned shiftdata=databits-bucketbits;
	unsigned databytes=(databits-1)/8u+1;

	if (verbose) {
		cerr << "Data words are " << databits << " bits long\n";
		cerr << "Data words are " << databytes << " bytes long\n";
		cerr << "Histogram contains " << nbuckets << " buckets\n";
		cerr << "Data shifted right by " << shiftdata << " bits\n";
	}

	unsigned long *histogram = new unsigned long [nbuckets];

	{ unsigned long i; for (i=0; i<nbuckets; ++i) histogram[i]=0; }

	unsigned long count=0;
	unsigned long largestcount=0;
	unsigned long largestcountindex;

	unsigned long minvalue=(unsigned long)((signed long)-1);
	unsigned long maxvalue=0;

	unsigned long minindex=nbuckets-1;
	unsigned long maxindex=0;

	while (1) {
		unsigned long value = readBigEndianValue(*instream,databytes);
		// cerr << "Read " << value << "\n";
		if (!instream->good()) break;

		if (value < minvalue) minvalue=value;
		if (value > maxvalue) maxvalue=value;

		unsigned long i = value >> shiftdata;
		if (i > nbuckets) {
			cerr << "Assertion failed - index > nbuckets\n";
			return 1;
		}

		if (i < minindex) minindex=i;
		if (i > maxindex) maxindex=i;

		if (histogram[i]+1 < histogram[i])
			cerr << "Warning - count overflow in " << i << "\n";
		else
			++histogram[i];

		if (histogram[i] > largestcount) {
			largestcount=histogram[i];
			largestcountindex=i;
		}
		++count;
	}

	if (verbose) {
		cerr << "Read " << count << " values\n";
		cerr << "Read " << count*databytes << " bytes\n";
		cerr << "Largest count " << largestcount
		     << " in index "     << largestcountindex
		     << " (value "       << (largestcountindex << shiftdata)
		     << ")\n";
		cerr << "Minimum value " << minvalue << "\n";
		cerr << "Maximum value " << maxvalue << "\n";
		cerr << "Minimum index " << minindex << "\n";
		cerr << "Maximum index " << maxindex << "\n";
	}

	if (showhist) {
		unsigned long i;
		for (i=minindex; i<=maxindex; ++i)
			cerr << "[" << i << "] = " << histogram[i] << "\n";
		cerr << "\n";
	}

	if (graphhist) {
		unsigned long i;
		for (i=minindex; i<=maxindex; ++i) {
			cerr << "[" << i << "] \t";
			unsigned j = (unsigned)(histogram[i]*40/largestcount);
			while (j--) cerr << "-";
			cerr << " " << (i << shiftdata)
			     << "*" << histogram[i] << "\n";
		}
		cerr << "\n";
	}

	return 0;
}

/*

gather statistics as we read ...

header must be skipped or removed ... could assume that the few strings and the like in the header would be overwhelmed by the image data, but the range would be upset by even one invalid value

could gather data in 4096 buckets using 16 bit value >> 4 as index:

	- if 16 bit data, >>4 spans 4096 buckets

	- if 12 bit data, >>4 spans 256 buckets

	- if 8 bit data >>4 spans 16 buckets ? enough to decide signed ??


if signed will be bimodal ... just above zero and below top

if unsigned center will be around middle value

calculate mean and standard deviation for both cases ...

	mean will be similar for signed and unsigned
	sd will be huge for signed

can deduce whether 8,12,16, particularly if signed, but depends on
actual dynamic range of data not the specified bumber of allocated bits

even if allocated bits are < 16, sign bit will still be 16 ? not necessarily
(eg. acr/nema 12 bit packed after unpacking may not sign extend).

once range is known, could rotate histogram to account for this and
recalculate stats

any overlays in the high bits will mess this up totally

what about byte order ? just try both ?

*/

