static char *CopyrightIdentifier(void) { return "@(#)wrwto8.cc 0.12 95/09/18 Copyright (C) 1993,1994,1995. David A. Clunie. All rights reserved."; }

#include <stdlib.h>		// for atoi()
#include <iostream.h>
#include <strstream.h>
#include <string.h>

#include "GetOpt.h"
#include "MainCmd.h"

int
main(int argc, char *argv[])
{
	CCOMMAND(argc,argv);

	cerr.setf(ios::showbase,ios::showbase);

	char		endian = 0;
	unsigned	width = 0;
	unsigned	height = 0;
	unsigned	depth = 8;
	unsigned	wordlength = 0;

	GetOpt getopt(argc,argv,"lbw:h:d:");
	int c;
	int err=0;
	while ((c=getopt()) != -1 && !err) {
		switch (c) {
			case 'l':	// little endian
			case 'b':	// big endian
				endian=c;
				break;
			case 'w':	// width
				width=atoi(getopt.optarg);
				break;
			case 'h':	// height
				height=atoi(getopt.optarg);
				break;
			case 'd':	// depth - defaults to 8
				depth=atoi(getopt.optarg);
				break;
			case '?':
			default:
				err=1;
		}
	}

	if (getopt.optind != argc) {
		cerr << "Trailing garbage after options\n" << flush;
		err=1;
	}

	switch (depth) {
		case 8:
			if (endian) {
				cerr << "Can't have byte order for 8 bits !\n"
				     << flush;
				err=1;
			}
			wordlength=1;
			break;
		case 12:
		case 16:
			if (!endian) {
				cerr << "Must specify byte order for "
				     << depth << " bit data\n"
				     << flush;
				err=1;
			}
			wordlength=2;
			break;
		default:
			cerr << "Unsupported data length " << depth << "\n"
			     << flush;
			err=1;
			break;
	}

	if (!width || !height) {
		cerr << "Must specify width & height\n" << flush;
		err=1;
	}

	if (err) {
		cerr << "Usage: " << argv[0] << " -w nn -h nn [-d nn -l|b]\n"
		     << flush;
		return 1;
	}

	unsigned i;
	for (i=0; i<width*height; ++i) {
		unsigned char c1,c2;
		cin.read(&c1,1);
		if (wordlength == 1) {
			cout.write(&c1,1);
		}
		else {
			cin.read(&c2,1);
			unsigned word;
			word=(endian == 'l') ? (c2<<8)+c1 : (c1<<8)+c2;
			c1=(char)(word>>(depth-8));
			cout.write(&c1,1);
		}
	}
	if (!cin.good()) {
		cerr << "Read error\n" << flush;
		return 1;
	}
	if (!cout.good()) {
		cerr << "Write error\n" << flush;
		return 1;
	}

	return 0;
}

