static char *CopyrightIdentifier(void) { return "@(#)wrwtopgm.cc 0.12 95/09/18 Copyright (C) 1993,1994,1995. David A. Clunie. All rights reserved."; }


#include <stdlib.h>		// for atoi()
#include <iostream.h>
#include <strstream.h>
#include <string.h>

#include "GetOpt.h"
#include "MainCmd.h"

int
main(int argc, char *argv[])
{
	CCOMMAND(argc,argv);

	cerr.setf(ios::showbase,ios::showbase);

	char		endian = 0;
	unsigned	width = 0;
	unsigned	height = 0;
	unsigned	depth = 8;
	unsigned	wordlength = 0;

	GetOpt getopt(argc,argv,"lbw:h:d:");
	int c;
	int err=0;
	while ((c=getopt()) != -1 && !err) {
		switch (c) {
			case 'l':	// little endian
			case 'b':	// big endian
				endian=c;
				break;
			case 'w':	// width
				width=atoi(getopt.optarg);
				break;
			case 'h':	// height
				height=atoi(getopt.optarg);
				break;
			case 'd':	// depth - defaults to 8
				depth=atoi(getopt.optarg);
				break;
			case '?':
			default:
				err=1;
		}
	}

	if (getopt.optind != argc) {
		cerr << "Trailing garbage after options\n" << flush;
		err=1;
	}

	switch (depth) {
		case 8:
			if (endian) {
				cerr << "Can't have byte order for 8 bits !\n"
				     << flush;
				err=1;
			}
			wordlength=1;
			break;
		case 12:
		case 16:
			if (!endian) {
				cerr << "Must specify byte order for "
				     << depth << " bit data\n"
				     << flush;
				err=1;
			}
			wordlength=2;
			break;
		default:
			cerr << "Unsupported data length " << depth << "\n"
			     << flush;
			err=1;
			break;
	}

	if (!width || !height) {
		cerr << "Must specify width & height\n" << flush;
		err=1;
	}

	if (err) {
		cerr << "Usage: " << argv[0] << " -l -b -w nn -h nn -d nn\n"
		     << flush;
		return 1;
	}

#if	PNMBIGRAW
	{
		unsigned long maxval = (1<<depth)-1;
		ostrstream ost;
		ost << "P5\n" << width << " " << height << "\n"
		    << maxval << "\n" << '\0';
		char *str=ost.str();
		cout.write(str,strlen(str));
		delete str;
	}

	unsigned i;
	for (i=0; i<width*height; ++i) {
		char c[2];
		cin.read(c,wordlength);
		// PNM format is little-endian ...
		if (wordlength == 2 && endian == 'b') {
			cout.write(&c[1],1);
			cout.write(&c[0],1);
		}
		else {
			cout.write(c,wordlength);
		}
	}
	if (!cin.good()) {
		cerr << "Read error\n" << flush;
		return 1;
	}
	if (!cout.good()) {
		cerr << "Write error\n" << flush;
		return 1;
	}

#else /* PNMBIGRAW */
	Compile Time Error - only extended raw PNM output supported
#endif /* PNMBIGRAW */

	return 0;
}

